﻿--Tipus utilitzats
CREATE TYPE decisio AS ENUM ('Aceptat', 'Denegat', 'Pendent');
CREATE TYPE nivellestudis AS ENUM ('Sense estudis', 'Primaria', 'Formacio profesional', 'Batxillerat', 'Estudis universitaris');
ALTER TYPE nivellestudis ADD VALUE 'Infantil'  BEFORE  'Primaria';
ALTER TYPE nivellestudis ADD VALUE 'ESO'  AFTER  'Primaria';
ALTER TYPE nivellestudis ADD VALUE 'Grau superior'
ALTER TYPE nivellestudis ADD VALUE 'Master'
ALTER TYPE nivellestudis ADD VALUE 'Doctorat'
CREATE TYPE tipusprocescaptacio AS ENUM ('Anunci radio','Anunci web');
--Esquema
CREATE SCHEMA uoc_recruitcat  AUTHORIZATION postgres;
--Taules
CREATE TABLE uoc_recruitcat.seuempresa
(
  codiseu character(10) NOT NULL,
  adreca character(100) NOT NULL,
  poblacio character(50) NOT NULL,
  codipostal character(5) NOT NULL,  
  datainaguracio date NOT NULL,
  nomresponsable character(50) NOT NULL,
  CONSTRAINT pk_seuempresa PRIMARY KEY (codiseu)
);
CREATE TABLE uoc_recruitcat.comercials
(
  codicomercial character(10)  NOT NULL,
  nom character(50) NOT NULL,
  codiseu character(10)  NOT NULL,
  CONSTRAINT pk_comercials PRIMARY KEY (codicomercial),
  CONSTRAINT fk_comercials_seuempresa FOREIGN KEY (codiseu)
      REFERENCES uoc_recruitcat.seuempresa (codiseu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
-- Index: uoc_recruitcat.fki_seuempresa
-- DROP INDEX uoc_recruitcat.fki_seuempresa;
CREATE INDEX fki_seuempresa
  ON uoc_recruitcat.comercials
  USING btree
  (codiseu);
CREATE TABLE uoc_recruitcat.clients
(
  codiclient character(10) NOT NULL,
  nom character(50) NOT NULL,
  adreca character(100) NOT NULL,
  poblacio character(50) NOT NULL,
  telefon character(15) NOT NULL,
  codiseu character(10) NOT NULL,
  comercialresponsable character(10)  NOT NULL,
  
  CONSTRAINT pk_clients PRIMARY KEY (codiclient),
  CONSTRAINT fk_clients_comercials FOREIGN KEY (comercialresponsable)
      REFERENCES uoc_recruitcat.comercials (codicomercial) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_clients_seuempresa FOREIGN KEY (codiseu)
      REFERENCES uoc_recruitcat.seuempresa (codiseu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
-- Index: uoc_recruitcat.fki_clients_comercials
-- DROP INDEX uoc_recruitcat.fki_clients_comercials;
CREATE INDEX fki_clients_comercials
  ON uoc_recruitcat.clients
  USING btree
  (comercialresponsable);
-- Index: uoc_recruitcat.fki_clients_seuempresa
-- DROP INDEX uoc_recruitcat.fki_clients_seuempresa;
CREATE INDEX fki_clients_seuempresa
  ON uoc_recruitcat.clients
  USING btree
  (codiseu);

CREATE TABLE uoc_recruitcat.visitacomercial
(
  codivisita character(10)  NOT NULL,
  data date NOT NULL,
  codicomercial character(10)  NOT NULL,
  codiclient character(20) NOT NULL,
  CONSTRAINT pk_visitacomercial PRIMARY KEY (codivisita),
  CONSTRAINT fk_visitacomercial_clients FOREIGN KEY (codiclient)
      REFERENCES uoc_recruitcat.clients (codiclient) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_visitacomercial_comercials FOREIGN KEY (codicomercial)
      REFERENCES uoc_recruitcat.comercials (codicomercial) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE uoc_recruitcat.processeleccio
(
  codiproces character(10)  NOT NULL,
  datainici date NOT NULL,
  datafi date NOT NULL,
  nivellestudis nivellestudis NOT NULL,
  codiclient character(10) NOT NULL,
  CONSTRAINT pk_processeleccio PRIMARY KEY (codiproces),
  CONSTRAINT pk_processeleccio_clients FOREIGN KEY (codiclient)
      REFERENCES uoc_recruitcat.clients (codiclient) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT chk_datainici_datafi CHECK (datainici < datafi)
);

CREATE TABLE uoc_recruitcat.candidats
(
  dni character(10) NOT NULL,
  nom character(50) NOT NULL,
  edat character(3) NOT NULL,
  nivellestudis nivellestudis NOT NULL,
  adreca character(100) NOT NULL,
  poblacio character(50) NOT NULL,
  telefon character(15),
  codiseu character(10)  NOT NULL,
  CONSTRAINT pk_candidats PRIMARY KEY (dni),
  CONSTRAINT fk_candidats_seuempresa FOREIGN KEY (codiseu)
      REFERENCES uoc_recruitcat.seuempresa (codiseu) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE uoc_recruitcat.entrevista
(
  codientrevista character(10)  NOT NULL,
  data date NOT NULL,
  notes text,
  codicandidat character(10) NOT NULL,
  CONSTRAINT pk_entrevista PRIMARY KEY (codientrevista),
  CONSTRAINT fk_entrevista_candidats FOREIGN KEY (codicandidat)
      REFERENCES uoc_recruitcat.candidats (dni) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE uoc_recruitcat.candidatsprocesseleccio
(
  codiproces character(10)  NOT NULL,
  codicandidat character(10) NOT NULL,
  decisio decisio NOT NULL,
  dataDecisio date NOT NULL,
  CONSTRAINT pk_candidatsprocesseleccio PRIMARY KEY (codiproces, codicandidat),
  CONSTRAINT fk_candidatsprocesseleccio_candidats FOREIGN KEY (codicandidat)
      REFERENCES uoc_recruitcat.candidats (dni) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_candidatsprocesseleecio_processeleccio FOREIGN KEY (codiproces)
      REFERENCES uoc_recruitcat.processeleccio (codiproces) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
-- Index: uoc_recruitcat.fki_candidatsprocesseleccio_candidats
-- DROP INDEX uoc_recruitcat.fki_candidatsprocesseleccio_candidats;
CREATE INDEX fki_candidatsprocesseleccio_candidats
  ON uoc_recruitcat.candidatsprocesseleccio
  USING btree
  (codicandidat);

CREATE TABLE uoc_recruitcat.competenciesprofesionals
(
  codicompetencia character(10)  NOT NULL,
  descripcio character(50) NOT NULL,
  datacreacio date NOT NULL,
  CONSTRAINT pk_competenciesprofesionals PRIMARY KEY (codicompetencia)
);

CREATE TABLE uoc_recruitcat.competenciespercandidat
(
  codicandidat character(10) NOT NULL,
  codicompetencia character(10)  NOT NULL,
  CONSTRAINT pk_competenciespercandidat PRIMARY KEY (codicandidat, codicompetencia),
  CONSTRAINT fk_competenciespercandidat_candidats FOREIGN KEY (codicandidat)
      REFERENCES uoc_recruitcat.candidats (dni) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_competenciespercandidat_competenciaprofesional FOREIGN KEY (codicompetencia)
      REFERENCES uoc_recruitcat.competenciesprofesionals (codicompetencia) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE uoc_recruitcat.processoscaptacio
(
  codiproces character(10)  NOT NULL,
  tipus tipusprocescaptacio NOT NULL,
  datainici date NOT NULL,
  CONSTRAINT pk_processoscaptacio PRIMARY KEY (codiproces)
);

CREATE TABLE uoc_recruitcat.estatprocescaptacio
(
  codiproces character(10)  NOT NULL,
  codicandidat character(10) NOT NULL,
  estat decisio NOT NULL,
  CONSTRAINT pk_estatprocescaptacio PRIMARY KEY (codiproces, codicandidat),
  CONSTRAINT fk_estatprocescaptacio_candidats FOREIGN KEY (codicandidat)
      REFERENCES uoc_recruitcat.candidats (dni) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_estatprocescaptacio_processoscaptacio FOREIGN KEY (codiproces)
      REFERENCES uoc_recruitcat.processoscaptacio (codiproces) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
-- Index: uoc_recruitcat.fki_estatprocescaptacio_candidats
-- DROP INDEX uoc_recruitcat.fki_estatprocescaptacio_candidats;
CREATE INDEX fki_estatprocescaptacio_candidats
  ON uoc_recruitcat.estatprocescaptacio
  USING btree
  (codicandidat);

CREATE TABLE uoc_recruitcat.propostespresentades
(
  codiproposta character(10) NOT NULL,
  import double precision NOT NULL,
  data date NOT NULL,
  estat decisio NOT NULL,
  codiclient character(10) NOT NULL,
  codicomercial character(10) NOT NULL,
  CONSTRAINT pk_propostespresentades PRIMARY KEY (codiproposta),
  CONSTRAINT fk_propostespresentades_clients FOREIGN KEY (codiclient)
      REFERENCES uoc_recruitcat.clients (codiclient) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk_propostespresentades_comercials FOREIGN KEY (codicomercial)
      REFERENCES uoc_recruitcat.comercials (codicomercial) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
CREATE TABLE uoc_recruitcat.competenciesperproces
(
  codiproces character(10) NOT NULL,
  codicompetencia character(10) NOT NULL,
  CONSTRAINT pk_competenciesperproces PRIMARY KEY (codiproces, codicompetencia)
);
