﻿--Repositori estadistic

--Est_a

--DROP TABLE uoc_recruitcat.est_a;
--Es crea la taula de la estadistica A
CREATE TABLE uoc_recruitcat.est_a
(
  anyentrevistes integer NOT NULL,
  numentrevistes integer NOT NULL
); 

CREATE OR REPLACE FUNCTION fn_update_est_a() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_cantitat integer; 

BEGIN
	--Es busca si el contador d'entrevistes existeix 
	SELECT numentrevistes INTO fn_cantitat FROM uoc_recruitcat.est_a WHERE anyentrevistes = EXTRACT(YEAR FROM NEW.data);
	IF FOUND THEN
		--Si existeix s'incrementa en una unitat
		UPDATE uoc_recruitcat.est_a SET numentrevistes =  fn_cantitat+1 WHERE anyentrevistes = EXTRACT(YEAR FROM NEW.data);
	ELSE
		--Si NO es crea el contador
		INSERT INTO uoc_recruitcat.est_a VALUES (EXTRACT(YEAR FROM NEW.data),1);
	END IF;
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Al introduir una nova entrevista s'afegeix un al contador d'entrevistes donat per la funció fn_update_est_a();
CREATE TRIGGER trg_update_est_a AFTER INSERT
ON uoc_recruitcat.entrevista FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_a();

--Delete
CREATE OR REPLACE FUNCTION fn_delete_est_a() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_cantitat integer; 

BEGIN
	--Es busca si el contador d'entrevistes existeix 
	SELECT numentrevistes INTO fn_cantitat FROM uoc_recruitcat.est_a WHERE anyentrevistes = EXTRACT(YEAR FROM OLD.data);
	IF FOUND THEN
		--Si existeix es decrementa en una unitat
		UPDATE uoc_recruitcat.est_a SET numentrevistes =  fn_cantitat-1 WHERE anyentrevistes = EXTRACT(YEAR FROM OLD.data);
	ELSE
		--Si NO es crea el contador a 0
		INSERT INTO uoc_recruitcat.est_a VALUES (EXTRACT(YEAR FROM OLD.data),0);
	END IF;
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Cada vegada que s'esborra una entrevista s'executa la funció fn_delete_est_a();
CREATE TRIGGER trg_delete_est_a AFTER DELETE
ON uoc_recruitcat.entrevista FOR EACH ROW
EXECUTE PROCEDURE fn_delete_est_a();


--est_b
DROP TABLE uoc_recruitcat.est_b;
--Es crea la taula de la estadistica B
CREATE TABLE uoc_recruitcat.est_b
(
  anyprocessos integer NOT NULL,
  cantitatprocessosseleccio integer NOT NULL,
  cantitatprocessosaceptats integer NOT NULL,
  percentatge real NOT NULL,
  CONSTRAINT pk_est_b PRIMARY KEY (anyprocessos)
);

CREATE OR REPLACE FUNCTION fn_update_est_b_candidatsprocessosseleccio() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_percent real;
fn_percent_temp real; 

BEGIN


--Es calcula el percentatge de processos de selecció que han acabat amb resultat positiu 
SELECT (CAST(count(*) AS FLOAT)/(SELECT count(DISTINCT codiproces) FROM uoc_recruitcat.candidatsprocesseleccio))*100 INTO fn_percent
FROM uoc_recruitcat.candidatsprocesseleccio 
WHERE decisio = 'Aceptat' AND EXTRACT(YEAR FROM datadecisio) = EXTRACT(YEAR FROM NEW.datadecisio);


--Es comprova si existeix el contador a la estadistica
SELECT percentatge INTO fn_percent_temp
FROM uoc_recruitcat.est_b WHERE anyprocessos = EXTRACT(YEAR FROM NEW.datadecisio);
	IF FOUND THEN	
		--Si existeix s'actualitza amb les dades calculades
		UPDATE uoc_recruitcat.est_b 
		SET cantitatprocessosseleccio =  0, percentatge = fn_percent 
		WHERE anyprocessos = EXTRACT(YEAR FROM NEW.datadecisio);
	ELSE
		--Si NO existeix es crea amb les dades a 0
		INSERT INTO uoc_recruitcat.est_b VALUES (EXTRACT(YEAR FROM NEW.datadecisio),0,0,0.00);
	END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Cada vegada que es produeix un canvi a la taula candidatsprocesseleccio es guarden les dades actualitzades executant la funció fn_update_est_b_candidatsprocessosseleccio().
CREATE TRIGGER trg_update_est_b_candidatsprocessosseleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.candidatsprocesseleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_b_candidatsprocessosseleccio();


--Est_c
--Es crea la taula de la estadistica C
CREATE TABLE uoc_recruitcat.est_c
(
  numcandidats integer NOT NULL,
  numaceptats integer NOT NULL,
  percentatgenoaceptats double precision NOT NULL,
  CONSTRAINT pk_est_c PRIMARY KEY (numcandidats)
);
--Es crea la funció que executarà el disprador trg_update_est_c_candidats
CREATE OR REPLACE FUNCTION fn_update_est_c_candidats() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_cantitat integer;
fn_cantitat_aceptats integer;
fn_cantitat_mai integer;
BEGIN

	--S'obté el nombre d'aceptats de la estadistica C
	SELECT numaceptats INTO fn_cantitat_aceptats
	FROM uoc_recruitcat.est_c;
	-- S'obté el nombre de candidats i es comprova si existeixen dades de la estadística C
	SELECT numcandidats INTO fn_cantitat
	FROM uoc_recruitcat.est_c;
	IF FOUND THEN
		--Si existeixen dades es suma 1 a la quantitat de processos de selecció
		fn_cantitat = fn_cantitat + 1;
		-- S'obtenen els que mai han estat seleccionats
		fn_cantitat_mai = fn_cantitat-fn_cantitat_aceptats;
		--S'actualitzen les dades amb les noves dades calculades del nombre de candidats no elegits mai
		UPDATE uoc_recruitcat.est_c 
		SET numcandidats =  fn_cantitat, 
		percentatgenoaceptats = (fn_cantitat_mai/CAST(fn_cantitat AS FLOAT))*100;
	ELSE
		--Si no existeixen dades s'inicialitzen les dades
		INSERT INTO uoc_recruitcat.est_c VALUES (1,0,100);
	END IF;
	
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disprador per la taula candidats que executa la funció fn_update_est_c_candidats()
CREATE TRIGGER trg_update_est_c_candidats AFTER INSERT
ON uoc_recruitcat.candidats FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_c_candidats();

--Es cre la funció que executarà el disprador trg_update_est_c_candidatsprocesseleccio
CREATE OR REPLACE FUNCTION fn_update_est_c_candidatsprocesseleccio() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_cantitat integer;
fn_cantitat_aceptats integer;
fn_cantitat_mai integer;

 

BEGIN

	--S'obté la quantitat de candidats acceptats
	SELECT 
	COUNT(DISTINCT(candidatsprocesseleccio.codicandidat)) INTO fn_cantitat_aceptats
	FROM 
	uoc_recruitcat.candidatsprocesseleccio
	WHERE decisio = 'Aceptat';
	-- S'obté el nombre de candidats de la estadística C
	SELECT numcandidats INTO fn_cantitat
	FROM uoc_recruitcat.est_c;
	IF FOUND THEN
		--Si es troben dades s'obté el nombre de candidats que mai han estat acceptats
		fn_cantitat_mai = fn_cantitat-fn_cantitat_aceptats;
		--S'actualitza la estadística amb les noves dades
		UPDATE uoc_recruitcat.est_c 
		SET numcandidats =  fn_cantitat, numaceptats = fn_cantitat_aceptats,  
		percentatgenoaceptats = (fn_cantitat_mai/CAST(fn_cantitat AS FLOAT))*100;
	ELSE
		--Si no existeixen dades s'inicialitzen les dades
		INSERT INTO uoc_recruitcat.est_c VALUES (0,1,0);
	END IF;
	
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disprador que exceutarà cada canvi de la taula candidatsprocesseleccio el procés fn_update_est_c_candidatsprocesseleccio()
CREATE TRIGGER trg_update_est_c_candidatsprocesseleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.candidatsprocesseleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_c_candidatsprocesseleccio();



--Est_d
DROP TABLE uoc_recruitcat.est_d;
--Es crea la taula de la estadistica D
CREATE TABLE uoc_recruitcat.est_d
(
  anyimport integer NOT NULL,
  importpropostesaceptades double precision,
  CONSTRAINT pk_est_d PRIMARY KEY (anyimport)
);

--Es crea la funció que executarà el disparador trg_update_est_d 
CREATE OR REPLACE FUNCTION fn_update_est_d() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_cantitat real;
fn_cantitat_temp real;
 

BEGIN

	--S'obtenen l'import de les propostes presentades aceptades 
	SELECT SUM(import) INTO fn_cantitat  FROM uoc_recruitcat.propostespresentades 
	WHERE estat = 'Aceptat' AND EXTRACT(YEAR FROM data) = EXTRACT(YEAR FROM NEW.data);
	
	--Es comrpova si existeixen dades a la estadística
	SELECT importpropostesaceptades INTO fn_cantitat_temp
	FROM uoc_recruitcat.est_d WHERE anyimport = EXTRACT(YEAR FROM NEW.data);
	IF FOUND THEN
		--Si existeixen dades s'actualitzen amb les noves dades
		UPDATE uoc_recruitcat.est_d 
		SET importpropostesaceptades =  fn_cantitat
		WHERE anyimport = EXTRACT(YEAR FROM NEW.data);
	ELSE
		--Si NO existeixen es creen les noves dades
		INSERT INTO uoc_recruitcat.est_d VALUES (EXTRACT(YEAR FROM NEW.data),NEW.import);
	END IF;
	
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disprador que executarà cada canvi a la taula propostespresentades el procés fn_update_est_d
CREATE TRIGGER trg_update_est_d AFTER INSERT OR UPDATE 
ON uoc_recruitcat.propostespresentades FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_d();


--Est_e
--Es crea la taula de la estadistica E temporal
CREATE TABLE uoc_recruitcat.est_e_temp
(
  codicomercial character(10) NOT NULL,
  propostesaceptades integer NOT NULL,
  propostespresentades integer NOT NULL,
  rati real NOT NULL,
  CONSTRAINT pk_est_e_temp PRIMARY KEY (codicomercial)
);
--Es crea la taula de la estadistica E
CREATE TABLE uoc_recruitcat.est_e
(
  codicomercial character(10) NOT NULL
);

--Es crea la funció que executarà el disparador trg_insert_est_e_propostespresentades
--Augmenta la quantita de propostes presentades i calcula el rati novament
CREATE OR REPLACE FUNCTION fn_insert_est_e_propostespresentades() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_codicomercial varchar(10);
fn_codicomercial_2 varchar(10);
fn_cantitat integer;
fn_cantitat_aceptats integer;
fn_cantitat_mai integer;
fn_rati real;
BEGIN
	--Obté la quantitat de acceptats del comercial del que es produeixen canvis de la taula temporal
	SELECT propostesaceptades INTO fn_cantitat_aceptats
	FROM uoc_recruitcat.est_e_temp
	WHERE codicomercial = NEW.codicomercial;

	--Obté la quantitat de propostes presentades del comercial del que es produeixen canvis de la taula temporal
	SELECT propostespresentades INTO fn_cantitat
	FROM uoc_recruitcat.est_e_temp
	WHERE codicomercial = NEW.codicomercial;	
	IF FOUND THEN
		--Si existeixen dades s'incrementen les propostes presentades
		fn_cantitat = fn_cantitat + 1;
		--Es calcula el rati
		fn_rati = (fn_cantitat_aceptats/CAST(fn_cantitat AS FLOAT))*100;
		--S'actualitzen les dades amb el nou calcul
		UPDATE uoc_recruitcat.est_e_temp 
		SET propostespresentades =  fn_cantitat,
		rati =  fn_rati
		WHERE codicomercial = NEW.codicomercial;
	ELSE
		--Si NO existeixen dades s'inicialitzen
		INSERT INTO uoc_recruitcat.est_e_temp VALUES (NEW.codicomercial,0,1,0);
	END IF;

--S'actualitza la taula on es guarda el comercial amb el millor percentatge d'exit

	--S'obté les dades del millor comercial
	SELECT codicomercial INTO fn_codicomercial FROM uoc_recruitcat.est_e_temp ORDER BY rati DESC LIMIT 1;
	--Es comprova si existeixen dades d'aquest comercial
	SELECT codicomercial INTO fn_codicomercial_2 FROM uoc_recruitcat.est_e;
	IF FOUND THEN
		--Si existeixen s'actualitzen les dades de la estadística.
		UPDATE uoc_recruitcat.est_e SET codicomercial = fn_codicomercial;
	ELSE
		--Si NO existeixen es creen de noves.
		INSERT INTO uoc_recruitcat.est_e VALUES(fn_codicomercial);
	END IF;	
	
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disparador que executarà cada canvi de la taula propostespresentadesla funció fn_insert_est_e_propostespresentades();
CREATE TRIGGER trg_insert_est_e_propostespresentades AFTER INSERT
ON uoc_recruitcat.propostespresentades FOR EACH ROW
EXECUTE PROCEDURE fn_insert_est_e_propostespresentades(); 

--Es crea la funció que executarà el disparador trg_update_est_e_propostesaceptades
--Calcula i actualitza la quantita de propostes acceptades
CREATE OR REPLACE FUNCTION fn_update_est_e_propostesaceptades() RETURNS TRIGGER AS $insertar$
DECLARE 
fn_codicomercial varchar(10);
fn_codicomercial_2 varchar(10);
fn_cantitat integer;
fn_cantitat_aceptats integer;
fn_cantitat_mai integer;
fn_rati real;
BEGIN
	--Si l'estat de la proposta es Acceptat s'actualitzen les dades
	IF(NEW.estat = 'Aceptat') THEN
	--S'obté les propostes aceptades de la taula temporal
	SELECT propostesaceptades INTO fn_cantitat_aceptats
	FROM uoc_recruitcat.est_e_temp
	WHERE codicomercial = NEW.codicomercial;
	--Obté la quantitat de propostes presentades de la taula temporal
	SELECT propostespresentades INTO fn_cantitat
	FROM uoc_recruitcat.est_e_temp
	WHERE codicomercial = NEW.codicomercial;	
	IF FOUND THEN
		--Si troba dades incrementa la quantitat de proppostes acceptades
		fn_cantitat_aceptats = fn_cantitat_aceptats + 1 ;
		--Torna a calcular el rati
		fn_rati = (fn_cantitat_aceptats/CAST(fn_cantitat AS FLOAT))*100;
		--Actualitza les dades de la taula temporal per aquest comercial.
		UPDATE uoc_recruitcat.est_e_temp 
		SET propostesaceptades =  fn_cantitat_aceptats,
		rati =  fn_rati
		WHERE codicomercial = NEW.codicomercial;
	ELSE
		--Si NO existeixen dades del comercial les crea.
		INSERT INTO uoc_recruitcat.est_e_temp VALUES (NEW.codicomercial,1,1,0);
	END IF;
	END IF;

	

--S'actualitza la taula on es guarda el comercial amb el millor percentatge d'exit

	--S'obté les dades del millor comercial
	SELECT codicomercial INTO fn_codicomercial FROM uoc_recruitcat.est_e_temp ORDER BY rati DESC LIMIT 1;
	--Es comrpova si existeixen dades a la estadítica
	SELECT codicomercial INTO fn_codicomercial_2 FROM uoc_recruitcat.est_e;
	IF FOUND THEN
		--Si existeixen s'actualitzen les dades de la estadística.
		UPDATE uoc_recruitcat.est_e SET codicomercial = fn_codicomercial;
	ELSE
		--Si NO es creen de noves.
		INSERT INTO uoc_recruitcat.est_e VALUES(fn_codicomercial);
	END IF;	
	
	RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disparador per executar cada canvi del taula propostespresentades la funció fn_update_est_e_propostesaceptades();
CREATE TRIGGER trg_update_est_e_propostesaceptades AFTER INSERT OR UPDATE
ON uoc_recruitcat.propostespresentades FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_e_propostesaceptades();

--Est_f
--Es crea la taula de la estadistica F
CREATE TABLE uoc_recruitcat.est_f
(
  processcaptaciomillor character(10) 
);

--Es crea la funció que executarà el disprador trg_update_est_f_candidatsprocesseleccio 
CREATE OR REPLACE FUNCTION fn_update_est_f_candidatsprocesseleccio() RETURNS TRIGGER AS $insertar$
DECLARE
fn_cantitat integer;
fn_proces_captacio varchar(10);
fn_proces_captacio_temp varchar(10);
BEGIN

--Es calculen les dades sobre quin és el millor procés de captació de candidats
SELECT 
   uoc_recruitcat.estatprocescaptacio.codiproces INTO fn_proces_captacio  
FROM 
  uoc_recruitcat.candidatsprocesseleccio, 
  uoc_recruitcat.estatprocescaptacio
WHERE 
  candidatsprocesseleccio.codicandidat = estatprocescaptacio.codicandidat AND
  candidatsprocesseleccio.decisio ='Aceptat'
GROUP BY estatprocescaptacio.codiproces
ORDER BY COUNT (candidatsprocesseleccio.codicandidat) DESC
LIMIT 1;

--Es comrpoven si existeixen dades a la estadística
SELECT processcaptaciomillor INTO fn_proces_captacio_temp FROM uoc_recruitcat.est_f;
IF FOUND THEN
	--Si existeixen dades s'actualitzen amb les noves dades
	UPDATE uoc_recruitcat.est_f SET processcaptaciomillor = fn_proces_captacio;
ELSE
	--Si NO existeixen es creen de noves
	INSERT INTO uoc_recruitcat.est_f VALUES(fn_proces_captacio);
END IF;


RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

-- Es crea el disparador que executarà en cada canvi de la taula candidatsprocesseleccio la funció fn_update_est_f_candidatsprocesseleccio
CREATE TRIGGER trg_update_est_f_candidatsprocesseleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.candidatsprocesseleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_f_candidatsprocesseleccio();


--Est_g
--Es crea la taula de la estadistica G
CREATE TABLE uoc_recruitcat.est_g
(
  nivellestudis nivellestudis
);

--Es crea la funció que executarà el disparador trg_update_est_g_processeleccio
CREATE OR REPLACE FUNCTION fn_update_est_g_processeleccio() RETURNS TRIGGER AS $insertar$
DECLARE
 mviews RECORD;
BEGIN
--S'esborren les dades anteriors a la estadística G
DELETE FROM uoc_recruitcat.est_g;
--Es crea un loop per obtenir les daes solicitades a la estadística
FOR mviews IN SELECT nivellestudis 
FROM uoc_recruitcat.processeleccio
WHERE EXTRACT (YEAR FROM datafi) = EXTRACT (YEAR FROM NEW.datafi)
GROUP BY nivellestudis 
ORDER BY count(*) DESC
LIMIT 10 LOOP
--Es guarden les dades cada pasada pel Loop, guardant així les 10 solicitades.
INSERT INTO uoc_recruitcat.est_g VALUES(mviews.nivellestudis);

END LOOP;



RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;
--Es crea el disprador que executarà cada canvi de la taula processeleccio la funció fn_update_est_g_processeleccio()
CREATE TRIGGER trg_update_est_g_processeleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.processeleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_g_processeleccio();

--Est h
--Es crea la taula de la estadistica H
CREATE TABLE uoc_recruitcat.est_h
(
  year integer,
  codicompetencia character(10)
);

--Es crea la funció que executarà el disprador trg_update_est_h_processeleccio
CREATE OR REPLACE FUNCTION fn_update_est_h_processeleccio() RETURNS TRIGGER AS $insertar$
DECLARE
 mviews RECORD;
BEGIN
--S'esborren les dades anteriors a la estadística H
DELETE FROM uoc_recruitcat.est_h WHERE year = EXTRACT(YEAR FROM NEW.datafi);
--Es crea un loop per obtenir les dades solicitades a la estadística
FOR mviews IN SELECT 
  codicompetencia 
FROM 
  uoc_recruitcat.competenciesperproces, 
  uoc_recruitcat.processeleccio
WHERE 
  processeleccio.codiproces = competenciesperproces.codiproces
  AND EXTRACT(YEAR FROM datafi) = EXTRACT (YEAR FROM NEW.datafi)
GROUP BY codicompetencia
ORDER BY COUNT (*) DESC
LIMIT 5 LOOP
--Es guarden les dades cada pasada pel Loop, guardant així les 5 solicitades.
INSERT INTO uoc_recruitcat.est_h VALUES(EXTRACT(YEAR FROM NEW.datafi),mviews.codicompetencia);

END LOOP;



RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;
--Es crea el disparador que executarà cada canvi de la taula processeleccio la funció fn_update_est_h_processeleccio();
CREATE TRIGGER trg_update_est_h_processeleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.processeleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_h_processeleccio();

--Est_i
--Es crea la taula de la estadistica I
CREATE TABLE uoc_recruitcat.est_i
(
  codiclient character(10)
);

--Es crea la funció que executarà el disprador trg_update_est_i_candidatsprocesseleccio
CREATE OR REPLACE FUNCTION fn_update_est_i_candidatsprocesseleccio() RETURNS TRIGGER AS $insertar$
DECLARE
fn_codi_client varchar(10);
fn_codi_client_temp varchar(10);
BEGIN

--Es calculen les dades del client que més candidats ens ha acceptat
SELECT 
  processeleccio.codiclient INTO fn_codi_client
FROM 
  uoc_recruitcat.candidatsprocesseleccio, 
  uoc_recruitcat.processeleccio
WHERE 
  processeleccio.codiproces = candidatsprocesseleccio.codiproces
  AND candidatsprocesseleccio.decisio = 'Aceptat'
GROUP BY processeleccio.codiclient
ORDER BY COUNT(processeleccio.codiclient) DESC
LIMIT 1;

--Es comprova si existeixen dades a la taula est_i
SELECT codiclient INTO fn_codi_client_temp FROM uoc_recruitcat.est_i; 
IF FOUND THEN
	--Si existeixen s'actualitzen
	UPDATE uoc_recruitcat.est_i SET codiclient = fn_codi_client;
ELSE
	--Si NO es creen de noves
	INSERT INTO uoc_recruitcat.est_i VALUES (fn_codi_client);
END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disparador que executarà cada canvi de la taula candidatsprocesseleccio la funció fn_update_est_i_candidatsprocesseleccio();
CREATE TRIGGER trg_update_est_i_candidatsprocesseleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.candidatsprocesseleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_i_candidatsprocesseleccio();

--Est_j
--Es crea la taula de la estadistica J
CREATE TABLE uoc_recruitcat.est_j
(
  anypropostes integer,
  codiclient character(10) 
);

--Es crea la funció que executarà el disparador trg_update_est_j_propostespresentades
CREATE OR REPLACE FUNCTION fn_update_est_j_propostespresentades() RETURNS TRIGGER AS $insertar$
DECLARE
fn_year integer;
fn_year_temp integer;
fn_codiclient varchar(10);
--fn_codiclient_temp varchar(10);
BEGIN
--S'obté l'any del càlcul
SELECT 
  EXTRACT (YEAR FROM t1.data) INTO fn_year 
FROM 
  uoc_recruitcat.propostespresentades t1
WHERE
  t1.estat = 'Denegat' AND EXTRACT (YEAR FROM t1.data) = EXTRACT (YEAR FROM NEW.data)
GROUP BY EXTRACT (YEAR FROM t1.data),t1.codiclient
ORDER BY COUNT (t1.codiclient) DESC
LIMIT 1;

--S'obté el client que més propostes ens ha denegat
SELECT      
  t1.codiclient INTO fn_codiclient 
FROM 
  uoc_recruitcat.propostespresentades t1
WHERE
  t1.estat = 'Denegat' AND EXTRACT (YEAR FROM t1.data) = EXTRACT (YEAR FROM NEW.data)
GROUP BY EXTRACT (YEAR FROM t1.data),t1.codiclient
ORDER BY COUNT (t1.codiclient) DESC
LIMIT 1;


--Es comprova si existeixen dades a la taula de la estadística
SELECT anypropostes INTO fn_year_temp FROM uoc_recruitcat.est_j WHERE anypropostes = fn_year;
IF FOUND THEN
	--Si es troben dades s'actualitzen amb les noves
	UPDATE uoc_recruitcat.est_j SET codiclient = fn_codiclient WHERE anypropostes = fn_year;
ELSE
	--Si NO es guarden de noves.
	INSERT INTO uoc_recruitcat.est_j VALUES (fn_year,fn_codiclient);
END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disparador que executarà cada canvi de la taula propostespresentades la funció fn_update_est_j_propostespresentades();
CREATE TRIGGER trg_update_est_j_propostespresentades
  AFTER INSERT OR UPDATE
  ON uoc_recruitcat.propostespresentades
  FOR EACH ROW
  EXECUTE PROCEDURE fn_update_est_j_propostespresentades();


-- Est_k
--Es crea la taula de la estadistica K
CREATE TABLE uoc_recruitcat.est_k
(
  percentatge double precision NOT NULL
);

--Es crea la funció que executarà el disparador trg_update_est_k_candidatsprocesseleccio
CREATE OR REPLACE FUNCTION fn_update_est_k_candidatsprocesseleccio() RETURNS TRIGGER AS $insertar$
DECLARE
fn_percentatge double precision;
fn_percentatge_temp double precision;
BEGIN

--S'obté el percentatge de candidats que mai han estat seleccionats per un client
SELECT (COUNT (candidats.dni)/CAST ((SELECT COUNT(*) FROM uoc_recruitcat.candidats) AS FLOAT) * 100) INTO fn_percentatge
FROM uoc_recruitcat.candidats
WHERE candidats.dni NOT IN (SELECT 
  DISTINCT dni  
FROM 
  uoc_recruitcat.candidats, 
  uoc_recruitcat.candidatsprocesseleccio
WHERE 
  candidats.dni = candidatsprocesseleccio.codicandidat);


--Es comprova si existeixen dades a la taula de la estadística K
SELECT percentatge INTO fn_percentatge_temp FROM uoc_recruitcat.est_k; 
IF FOUND THEN
	--Si existeixen, s'actualitzen amb les noves dades.
	UPDATE uoc_recruitcat.est_k SET percentatge = fn_percentatge;
ELSE
	--Si NO existeixen es creen de noves
	INSERT INTO uoc_recruitcat.est_k VALUES (fn_percentatge);
END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea el disparador que executarà cada canvi de la taula candidatsprocesseleccio la funció fn_update_est_k_candidatsprocesseleccio();
CREATE TRIGGER trg_update_est_k_candidatsprocesseleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.candidatsprocesseleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_k_candidatsprocesseleccio();



--Est_l
--Es crea la taula de la estadistica L
CREATE TABLE uoc_recruitcat.est_l
(
  codicomercial character(10) NOT NULL
) ;

--Es crea la funció que executarà el disparador trg_update_est_l_visitacomercial
CREATE OR REPLACE FUNCTION fn_update_est_l_visitacomercial() RETURNS TRIGGER AS $insertar$
DECLARE
fn_codi_comercial varchar(10);
fn_codi_comercial_temp varchar(10);
BEGIN

--Obtenim el comercial que més visites ha realitzat en l'any en curs
SELECT 
  visitacomercial.codicomercial INTO fn_codi_comercial
FROM 
  uoc_recruitcat.visitacomercial 
WHERE EXTRACT (YEAR FROM visitacomercial.data) = EXTRACT (YEAR FROM current_date) 
GROUP BY visitacomercial.codicomercial
ORDER BY COUNT (visitacomercial.codicomercial) DESC
LIMIT 1;

--Comprovem si existeixen dades a la estadística l
SELECT codicomercial INTO fn_codi_comercial_temp FROM uoc_recruitcat.est_l; 
IF FOUND THEN
	--Si existeixen actualitzem les dades
	UPDATE uoc_recruitcat.est_l SET codicomercial = fn_codi_comercial;
ELSE
	--Si NO existeixen es creen de noves
	INSERT INTO uoc_recruitcat.est_l VALUES (fn_codi_comercial);
END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea un disparador per la taula visitacomercial que executa la funció fn_update_est_l_visitacomercial
CREATE TRIGGER trg_update_est_l_visitacomercial AFTER INSERT OR UPDATE
ON uoc_recruitcat.visitacomercial FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_l_visitacomercial();

--Est_m
--Es crea la taula de la estadistica M
CREATE TABLE uoc_recruitcat.est_m
(
  codiproces character(10) 
);

--Es crea la funció que executarà el disparador trg_update_est_m_candidatsprocesseleccio
CREATE OR REPLACE FUNCTION fn_update_est_m_candidatsprocesseleccio() RETURNS TRIGGER AS $insertar$
DECLARE
fn_codi_proces varchar(10);
fn_codi_proces_temp varchar(10);
BEGIN

--Es calcula el procés de captació que ens ha proporcionat més candidats
SELECT 
  estatprocescaptacio.codiproces INTO fn_codi_proces
FROM 
  uoc_recruitcat.estatprocescaptacio, 
  uoc_recruitcat.candidats, 
  uoc_recruitcat.candidatsprocesseleccio
WHERE 
  estatprocescaptacio.codicandidat = candidats.dni AND
  candidatsprocesseleccio.codicandidat = candidats.dni AND
  candidatsprocesseleccio.decisio = 'Aceptat' AND
  EXTRACT(YEAR FROM candidatsprocesseleccio.datadecisio) = EXTRACT(YEAR FROM current_date)
GROUP BY estatprocescaptacio.codiproces
ORDER BY COUNT(estatprocescaptacio.codiproces) DESC
LIMIT 1;
--Es comprova si existeixen dades a la estadística
SELECT codiproces INTO fn_codi_proces_temp FROM uoc_recruitcat.est_m; 
IF FOUND THEN
	--Si es troben dades s'actualitzen amb les noves dades.
	UPDATE uoc_recruitcat.est_m SET codiproces = fn_codi_proces;
ELSE
	--Si NO es troben es creen de noves.
	INSERT INTO uoc_recruitcat.est_m VALUES (fn_codi_proces);
END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

--Es crea un disparador per la taula candidatsprocesseleccio per executar el procediment fn_update_est_m_candidatsprocesseleccio()
CREATE TRIGGER trg_update_est_m_candidatsprocesseleccio AFTER INSERT OR UPDATE
ON uoc_recruitcat.candidatsprocesseleccio FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_m_candidatsprocesseleccio();

--Est_n
--Es crea la taula de la estadistica N
CREATE TABLE uoc_recruitcat.est_n
(
  codiseu character(10) NOT NULL
);

--Es crea la funció que executarà el disparador trg_update_est_n_entrevista
CREATE OR REPLACE FUNCTION fn_update_est_n_entrevista() RETURNS TRIGGER AS $insertar$
DECLARE
fn_codi_seu varchar(10);
fn_codi_seu_temp varchar(10);
BEGIN

--S'obtenen les dades de la seu amb menys entrevistes
SELECT 
  candidats.codiseu INTO fn_codi_seu  
FROM 
  uoc_recruitcat.candidats, 
  uoc_recruitcat.entrevista
WHERE 
  candidats.dni = entrevista.codicandidat AND
  EXTRACT (YEAR FROM entrevista.data) = EXTRACT (YEAR FROM current_date)
GROUP BY (candidats.codiseu)
ORDER BY COUNT (entrevista.codientrevista) ASC
LIMIT 1;


--Es comprova si existeixen dades a la estadística
SELECT codiseu INTO fn_codi_seu_temp FROM uoc_recruitcat.est_n; 
IF FOUND THEN
	--Si es troben dades s'actualitzen amb les noves dades.
	UPDATE uoc_recruitcat.est_n SET codiseu = fn_codi_seu;
ELSE
	--Si NO es troben es creen de noves.
	INSERT INTO uoc_recruitcat.est_n VALUES (fn_codi_seu);
END IF;

RETURN NULL;

END;
$insertar$ LANGUAGE plpgsql;

-- Es crea un disparador per la taula entrevista per executar el procediment fn_update_est_n_entrevista()
CREATE TRIGGER trg_update_est_n_entrevista AFTER INSERT OR UPDATE
ON uoc_recruitcat.entrevista FOR EACH ROW
EXECUTE PROCEDURE fn_update_est_n_entrevista();
